using System;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win.FormattedLinkLabel;
using Infragistics.Win.UltraWinScrollBar;

namespace Infragistics.Win.FormattedLinkLabel
{
    public class UltraFormattedTextEditorEx : UltraFormattedTextEditor
    {
        public ScrollBarUIElement HScrollBar;
        public ScrollBarUIElement VScrollBar;

        public UltraFormattedTextEditorEx()
        {
            MouseUp += new MouseEventHandler(OnMouseUp);
            MouseMove += new MouseEventHandler(OnMouseMove);
            this.PreviewKeyDown += new PreviewKeyDownEventHandler(UltraFormattedTextEditorEx_PreviewKeyDown);
            this.Paint += new PaintEventHandler(UltraFormattedTextEditorEx_Paint);
        }


        void UltraFormattedTextEditorEx_Paint(object sender, PaintEventArgs e)
        {
            FireHScrollEvents();
            FireVScrollEvents();
        }

        void UltraFormattedTextEditorEx_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            FireHScrollEvents();
            FireVScrollEvents();
        }

        private void OnMouseUp(object sender, MouseEventArgs e)
        {
            //if (e.Button == System.Windows.Forms.MouseButtons.Left)
            FireHScrollEvents();
            FireVScrollEvents();
        }
        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            FireHScrollEvents();
            FireVScrollEvents();
        }
        private void FireHScrollEvents()
        {
            HScrollBar = this.UIElement.ChildElements[0].ChildElements[0].ChildElements[2] as ScrollBarUIElement;
            if (HScrollBar == null)
                return;

            hNewValue = Convert.ToInt32((HScrollBar.AccessibilityInstance).Value);
            if (HScrollBar.Enabled && (hOldValue != hNewValue))
            {
                ScrollEventArgs scaHoriz = new ScrollEventArgs(ScrollEventType.ThumbPosition, hOldValue, hNewValue, ScrollOrientation.HorizontalScroll);
                if (HScroll != null) HScroll(this, scaHoriz);
                hOldValue = hNewValue;
            }
        }

        private void FireVScrollEvents()
        {
            VScrollBar = this.UIElement.ChildElements[0].ChildElements[0].ChildElements[1] as ScrollBarUIElement;
            if (VScrollBar == null)
                return;

            vNewValue = Convert.ToInt32((VScrollBar.AccessibilityInstance).Value);
            if (VScrollBar.Enabled && (vOldValue != vNewValue))
            {
                ScrollEventArgs scaVert = new ScrollEventArgs(ScrollEventType.ThumbPosition, vOldValue, vNewValue, ScrollOrientation.VerticalScroll);
                if (VScroll != null) VScroll(this, scaVert);
                vOldValue = vNewValue;
            }
        }


        public delegate void ChangingHandler(object sender, ScrollEventArgs scrollEventArgs);

        public event ChangingHandler HScroll;
        public event ChangingHandler VScroll;

        private int hOldValue;
        private int hNewValue;

        private int vOldValue;
        private int vNewValue;

    }
}
